@@ -25,6 +25,8 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 
+#include <signal.h>
+
 #include "main-window.h"
 
 #include "vfs-file-info.h"
@@ -36,105 +38,151 @@
 #include "vfs-thumbnail-loader.h"
 
 #include "ptk-utils.h"
-#include "app-chooser-dialog.h"
+#include "ptk-app-chooser.h"
+#include "ptk-file-properties.h"
+#include "ptk-file-menu.h"
 
 #include "settings.h"
-#include "fm-desktop.h"
+
+#include "desktop.h"
+
+typedef enum{
+    CMD_OPEN = 1,
+    CMD_OPEN_TAB,
+    CMD_FILE_PROP,
+    CMD_FILE_MENU,
+    CMD_DAEMON_MODE
+}SocketEvent;
+
+static gboolean initialized = FALSE;
 
 static int sock;
 GIOChannel* io_channel = NULL;
 
+gboolean daemon_mode = FALSE;
+
 static char* default_files[2] = {NULL, NULL};
 static char** files = NULL;
 static gboolean no_desktop = FALSE;
 static gboolean old_show_desktop = FALSE;
 
+static gboolean new_tab = FALSE;
+static gboolean file_prop = FALSE;
+static gboolean file_menu = FALSE;
+
+#ifdef HAVE_HAL
 static char* mount = NULL;
 static char* umount = NULL;
 static char* eject = NULL;
+#endif
 
 static GOptionEntry opt_entries[] =
 {
     { "no-desktop", '\0', 0, G_OPTION_ARG_NONE, &no_desktop, N_("Don't show desktop icons."), NULL },
+    { "daemon-mode", 'd', 0, G_OPTION_ARG_NONE, &daemon_mode, N_("Run PCManFM as a daemon"), NULL },
+    { "new-tab", 't', 0, G_OPTION_ARG_NONE, &new_tab, N_("Open folders in new tabs of the last used window instead of creating new windows"), NULL },
+/*    { "file-prop", 'p', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &file_prop, NULL, NULL },
+    { "file-menu", 'n', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &file_menu, NULL, NULL }, */
 #ifdef HAVE_HAL
-/*
-    { "mount", 'm', 0, G_OPTION_ARG_STRING, &mount, N_("Mount specified device"), "device [mount options]" },
-    { "umount", 'u', 0, G_OPTION_ARG_STRING, &umount, N_("Unmount specified device"), "device" },
-    { "eject", 'e', 0, G_OPTION_ARG_STRING, &eject, N_("Eject specified device"), "device" },
-*/
+    /* hidden arguments used to mount volumes */
+    { "mount", 'm', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &mount, NULL, NULL },
+    { "umount", 'u', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &umount, NULL, NULL },
+    { "eject", 'e', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_STRING, &eject, NULL, NULL },
 #endif
     {G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &files, NULL, N_("[FILE1, FILE2,...]")},
     { NULL }
 };
 
-static FMMainWindow* create_main_window();
+static gboolean single_instance_check();
+static void single_instance_finalize();
+static void get_socket_name( char* buf, int len );
+static gboolean on_socket_event( GIOChannel* ioc, GIOCondition cond, gpointer data );
 
+static void init_folder();
+static void check_icon_theme();
+
+static gboolean handle_parsed_commandline_args();
+
+static FMMainWindow* create_main_window();
 static void open_file( const char* path );
 
-static gboolean on_socket_event( GIOChannel* ioc,
-                                 GIOCondition cond,
-                                 gpointer data )
+static GList* get_file_info_list( char** files );
+
+gboolean on_socket_event( GIOChannel* ioc, GIOCondition cond, gpointer data )
 {
     int client, r;
     socklen_t addr_len = 0;
-    GString* paths;
-    struct sockaddr_un client_addr =
-        {
-            0
-        };
-    FMMainWindow* main_window;
-    static char buf[ 256 ];
+    struct sockaddr_un client_addr ={ 0 };
+    static char buf[ 1024 ];
+    GString* args;
+    char** file;
+    SocketEvent cmd;
 
     if ( cond & G_IO_IN )
     {
         client = accept( g_io_channel_unix_get_fd( ioc ), (struct sockaddr *)&client_addr, &addr_len );
         if ( client != -1 )
         {
-            char *path, *end;
-            paths = g_string_new_len( NULL, 512 );
+            args = g_string_new_len( NULL, 2048 );
             while( (r = read( client, buf, sizeof(buf) )) > 0 )
-                g_string_append_len( paths, buf, r);
+                g_string_append_len( args, buf, r);
             shutdown( client, 2 );
             close( client );
 
-            for( end = path = paths->str; path && *path;  )
+            new_tab = FALSE;
+            file_prop = FALSE;
+            file_menu = FALSE;
+
+            switch( args->str[0] )
             {
-                end = strchr(path, '\n');
-                if( end )
-                    *end = '\0';
-
-                GDK_THREADS_ENTER();
-
-                if ( g_file_test( path, G_FILE_TEST_IS_DIR ) )
-                {
-                    main_window = create_main_window();
-                    fm_main_window_add_new_tab( main_window, path,
-                                                appSettings.showSidePane,
-                                                appSettings.sidePaneMode );
-                    gtk_window_present( GTK_WINDOW(main_window) );
-                }
-                else
-                {
-                    open_file( path );
-                }
-                GDK_THREADS_LEAVE();
-
-                path = end ? (end + 1) : NULL;
+            case CMD_OPEN_TAB:
+                new_tab = TRUE;
+                break;
+            case CMD_FILE_PROP:
+                file_prop = TRUE;
+                break;
+            case CMD_FILE_MENU:
+                file_menu = TRUE;
+                break;
+            case CMD_DAEMON_MODE:
+                daemon_mode = TRUE;
+                g_string_free( args, TRUE );
+                return TRUE;
             }
-            g_string_free( paths, TRUE );
+
+            if( args->str[ 1 ] )
+                files = g_strsplit( args->str + 1, "\n", 0 );
+            else
+                files = NULL;
+            g_string_free( args, TRUE );
+
+            GDK_THREADS_ENTER();
+
+            if( files )
+            {
+                for( file = files; *file; ++file )
+                {
+                    if( ! **file )  /* remove empty string at tail */
+                        *file = NULL;
+                }
+            }
+            handle_parsed_commandline_args();
+
+            GDK_THREADS_LEAVE();
         }
     }
+
     return TRUE;
 }
 
-static void get_socket_name( char* buf, int len )
+void get_socket_name( char* buf, int len )
 {
     char* dpy = gdk_get_display();
     g_snprintf( buf, len, "/tmp/.pcmanfm-socket%s-%s", dpy, g_get_user_name() );
     g_free( dpy );
 }
 
-static void single_instance_init()
+gboolean single_instance_check()
 {
     struct sockaddr_un addr;
     int addr_len;
@@ -156,36 +204,40 @@ static void single_instance_init()
     addr_len = strlen( addr.sun_path ) + sizeof( addr.sun_family );
 #endif
 
+    /* try to connect to existing instance */
     if ( connect( sock, ( struct sockaddr* ) & addr, addr_len ) == 0 )
     {
-        char** file;
         /* connected successfully */
-        if( ! files )
+        char** file;
+        char cmd = CMD_OPEN;
+
+        if( daemon_mode )
+            cmd = CMD_DAEMON_MODE;
+        else if( new_tab )
+            cmd = CMD_OPEN_TAB;
+        else if( file_prop )
+            cmd = CMD_FILE_PROP;
+        else if( file_menu )
+            cmd = CMD_FILE_MENU;
+
+        write( sock, &cmd, sizeof(char) );
+
+        if( files )
         {
-            files = default_files;
-            files[0] = (char *) g_get_home_dir();
-        }
-        for( file = files; *file; ++file )
-        {
-            char* path = *file;
-            if( g_str_has_prefix( path, "file:" ) ) /* It's a URI */
+            for( file = files; *file; ++file )
             {
-                path = g_filename_from_uri( path, NULL, NULL );
-                g_free( *file );
-                *file = path;
+                write( sock, *file, strlen( *file ) );
+                write( sock, "\n", 1 );
             }
-            write( sock, path, strlen( path ) );
-            write( sock, "\n", 1 );
         }
+
         shutdown( sock, 2 );
         close( sock );
-        gdk_notify_startup_complete();
-
         ret = 0;
         goto _exit;
     }
 
-    /* There is no existing server. So, we are in the first instance. */
+    /* There is no existing server, and we are in the first instance. */
     unlink( addr.sun_path ); /* delete old socket file if it exists. */
     reuse = 1;
     ret = setsockopt( sock, SOL_SOCKET, SO_REUSEADDR, &reuse, sizeof( reuse ) );
@@ -210,10 +262,12 @@ static void single_instance_init()
     return ;
 
 _exit:
+
+    gdk_notify_startup_complete();
     exit( ret );
 }
 
-static void single_instance_finalize()
+void single_instance_finalize()
 {
     char lock_file[ 256 ];
 
@@ -229,8 +283,8 @@ FMMainWindow* create_main_window()
 {
     FMMainWindow * main_window = FM_MAIN_WINDOW(fm_main_window_new ());
     gtk_window_set_default_size( GTK_WINDOW( main_window ),
-                                 appSettings.width, appSettings.height );
-    if ( appSettings.maximized )
+                                 app_settings.width, app_settings.height );
+    if ( app_settings.maximized )
     {
         gtk_window_maximize( GTK_WINDOW( main_window ) );
     }
@@ -238,7 +292,7 @@ FMMainWindow* create_main_window()
     return main_window;
 }
 
-static void check_icon_theme()
+void check_icon_theme()
 {
     GtkSettings * settings;
     char* theme;
@@ -305,48 +359,191 @@ static void _debug_gdk_threads_leave()
 }
 #endif
 
-static void init_folder()
+void init_folder()
 {
-    appSettings.bookmarks = ptk_bookmarks_get();
+    if( G_LIKELY(initialized) )
+        return;
+
+    app_settings.bookmarks = ptk_bookmarks_get();
 
     vfs_volume_init();
     vfs_thumbnail_init();
 
-    vfs_mime_type_set_icon_size( appSettings.bigIconSize,
-                                 appSettings.smallIconSize );
-    vfs_file_info_set_thumbnail_size( appSettings.bigIconSize,
-                                      appSettings.smallIconSize );
+    vfs_mime_type_set_icon_size( app_settings.big_icon_size,
+                                 app_settings.small_icon_size );
+    vfs_file_info_set_thumbnail_size( app_settings.big_icon_size,
+                                      app_settings.small_icon_size );
 
     check_icon_theme();
+    initialized = TRUE;
 }
 
 #ifdef HAVE_HAL
+
+/* FIXME: Currently, this cannot be supported without HAL */
+
 static int handle_mount( char** argv )
 {
+    gboolean success;
     vfs_volume_init();
     if( mount )
-    {
-
-    }
+        success = vfs_volume_mount_by_udi( mount, NULL );
     else if( umount )
-    {
-
-    }
+        success = vfs_volume_umount_by_udi( umount, NULL );
     else /* if( eject ) */
-    {
-
-    }
+        success = vfs_volume_eject_by_udi( eject, NULL );
     vfs_volume_finalize();
-    return 0;
+    return success ? 0 : 1;
 }
 #endif
 
-int
-main ( int argc, char *argv[] )
+GList* get_file_info_list( char** file_paths )
+{
+    GList* file_list = NULL;
+    char** file;
+    VFSFileInfo* fi;
+
+    for( file = file_paths; *file; ++file )
+    {
+        fi = vfs_file_info_new();
+        if( vfs_file_info_get( fi, *file, NULL ) )
+            file_list = g_list_append( file_list, fi );
+        else
+            vfs_file_info_unref( fi );
+    }
+
+    return file_list;
+}
+
+gboolean delayed_popup( GtkWidget* popup )
+{
+    GDK_THREADS_ENTER();
+
+    gtk_menu_popup( GTK_MENU( popup ), NULL, NULL,
+                    NULL, NULL, 0, gtk_get_current_event_time() );
+
+    GDK_THREADS_LEAVE();
+
+    return FALSE;
+}
+
+gboolean handle_parsed_commandline_args()
 {
     FMMainWindow * main_window = NULL;
-    GOptionContext *context;
     char** file;
+
+    /* If no files are specified, open home dir by defualt. */
+    if( G_LIKELY( ! files ) )
+    {
+        files = default_files;
+        files[0] = (char *) g_get_home_dir();
+    }
+
+    /* get the last active window, if available */
+    if( new_tab )
+    {
+        main_window = fm_main_window_get_last_active();
+    }
+    else if( file_menu )    /* show popup menu for files */
+    {
+        /* FIXME: This doesn't work properly */
+        GtkMenu* popup;
+        GList* file_list;
+
+        if( ! daemon_mode )
+        {
+            g_warning( "--file-menu is only availble when pcmanfm daemon is running." );
+            return FALSE;
+        }
+
+        file_list = get_file_info_list( files );
+
+        if( file_list )
+        {
+            char* dir_name = g_path_get_dirname( files[0] );
+            popup = ptk_file_menu_new(
+                        files[0], (VFSFileInfo*)file_list->data,
+                        dir_name,
+                        file_list, NULL );
+            /* FIXME: I have no idea why this crap is needed.
+             *  Without this delay, the menu will fail to popup. */
+            g_timeout_add( 150, (GSourceFunc)delayed_popup, popup );
+        }
+
+        if( files != default_files )
+            g_free( files );
+        files = NULL;
+        return TRUE;
+    }
+    else if( file_prop )    /* show file properties dialog */
+    {
+        GList* file_list;
+
+        file_list = get_file_info_list( files );
+        if( file_list )
+        {
+            GtkWidget* dlg;
+            char* dir_name = g_path_get_dirname( files[0] );
+            dlg = file_properties_dlg_new( NULL, dir_name, file_list );
+            gtk_dialog_run( dlg );
+            gtk_widget_destroy( dlg );
+            g_free( dir_name );
+
+            if( files != default_files )
+                g_free( files );
+            return FALSE;
+        }
+    }
+
+    /* open files passed in command line arguments */
+    for( file = files; *file; ++file )
+    {
+        char* file_path, *real_path;
+
+        if( ! **file )  /* skip empty string */
+            continue;
+
+        if( g_str_has_prefix( *file, "file:" ) ) /* It's a URI */
+        {
+            file_path = g_filename_from_uri( *file, NULL, NULL );
+            g_free( *file );
+            *file = file_path;
+        }
+        else
+            file_path = *file;
+
+        real_path = vfs_file_resolve_path( NULL, file_path );
+        if( g_file_test( real_path, G_FILE_TEST_IS_DIR ) )
+        {
+            if( G_UNLIKELY( ! main_window ) )   /* create main window if needed */
+            {
+                /* initialize things required by folder view... */
+                if( G_UNLIKELY( ! daemon_mode ) )
+                    init_folder();
+                main_window = create_main_window();
+            }
+            fm_main_window_add_new_tab( main_window, real_path,
+                                        app_settings.show_side_pane,
+                                        app_settings.side_pane_mode );
+        }
+        else
+        {
+            open_file( real_path );
+        }
+        g_free( real_path );
+    }
+
+    if( files != default_files )
+        g_strfreev( files );
+
+    files = NULL;
+
+    return TRUE;
+}
+
+int main ( int argc, char *argv[] )
+{
+    gboolean run = FALSE;
     GError* err = NULL;
 
 #ifdef ENABLE_NLS
@@ -363,18 +560,9 @@ main ( int argc, char *argv[] )
     g_thread_init( NULL );
     gdk_threads_init ();
 
-    /* parse command line arguments */
-    context = g_option_context_new ("");
-    g_option_context_add_main_entries (context, opt_entries, GETTEXT_PACKAGE);
-    g_option_context_add_group (context, gtk_get_option_group (TRUE));
-    /* gtk_init( &argc, &argv ); is not needed if g_option_context_parse is called */
-    if( G_UNLIKELY( ! g_option_context_parse (context, &argc, &argv, &err) ) )
-    {
-        g_print( _("Error: %s\n"), err->message );
-        g_error_free( err );
+    /* initialize GTK+ and parse the command line arguments */
+    if( G_UNLIKELY( ! gtk_init_with_args( &argc, &argv, "", opt_entries, GETTEXT_PACKAGE, &err ) ) )
         return 1;
-    }
-    g_option_context_free( context );
 
 #if HAVE_HAL
     /* If the user wants to mount/umount/eject a device */
@@ -382,32 +570,14 @@ main ( int argc, char *argv[] )
         return handle_mount( argv );
 #endif
 
+    /* ensure that there is only one instance of pcmanfm.
+         if there is an existing instance, command line arguments
+         will be passed to the existing instance, and exit() will be called here.  */
+    single_instance_check( argc - 1, argv + 1 );
+
     load_settings();    /* load config file */
 
-    /* if ( appSettings.singleInstance ) */
-    single_instance_init(); /* ensure there is only one instance of pcmanfm. */
-
-    /* temporarily turn off desktop if needed */
-    if( G_LIKELY( no_desktop ) )
-    {
-        /* No matter what the value of showDesktop is, we don't showdesktop icons
-         * if --no-desktop argument is passed by the users. */
-        old_show_desktop = appSettings.showDesktop;
-        /* This config value will be restored before saving config files, if needed. */
-        appSettings.showDesktop = FALSE;
-    }
-
-    /* If no command line arguments are specified, open home dir by defualt. */
-    if( G_LIKELY( ! files ) )
-    {
-        files = default_files;
-        /* only show home dir when desktop icons is not used. */
-        if( ! appSettings.showDesktop )
-            files[0] = (char *) g_get_home_dir();
-    }
-
-    vfs_file_info_set_utf8_filename( g_get_filename_charsets( NULL ) );
-
+    /* initialize the file alteration monitor */
     if( G_UNLIKELY( ! vfs_file_monitor_init() ) )
     {
         ptk_show_error( NULL, _("Error"), _("Error: Unable to establish connection with FAM.\n\nDo you have \"FAM\" or \"Gamin\" installed and running?") );
@@ -416,87 +586,68 @@ main ( int argc, char *argv[] )
         return 1;
     }
 
-    vfs_mime_type_init();   /* Initialize our mime-type system */
+    /* check if the filename encoding is UTF-8 */
+    vfs_file_info_set_utf8_filename( g_get_filename_charsets( NULL ) );
 
-    /* open files passed in command line arguments */
-    for( file = files; *file; ++file )
+    /* Initialize our mime-type system */
+    vfs_mime_type_init();
+
+    /* temporarily turn off desktop if needed */
+    if( G_LIKELY( no_desktop ) )
     {
-        char* file_path, *real_path;
-        if( g_str_has_prefix( *file, "file:" ) ) /* It's a URI */
-        {
-            file_path = g_filename_from_uri( *file, NULL, NULL );
-            g_free( *file );
-            *file = file_path;
-        }
-        else
-            file_path = *file;
-
-        real_path = vfs_file_resolve_path( NULL, file_path );
-        if( g_file_test( real_path, G_FILE_TEST_IS_DIR ) )
-        {
-            if( G_UNLIKELY( ! main_window ) )   /* create main window if needed */
-            {
-                /* initialize things required by folder view... */
-                init_folder();
-                main_window = create_main_window();
-            }
-            fm_main_window_add_new_tab( main_window, real_path,
-                                        appSettings.showSidePane,
-                                        appSettings.sidePaneMode );
-        }
-        else
-        {
-            open_file( real_path );
-        }
-        g_free( real_path );
+        /* No matter what the value of show_desktop is, we don't showdesktop icons
+         * if --no-desktop argument is passed by the users. */
+        old_show_desktop = app_settings.show_desktop;
+        /* This config value will be restored before saving config files, if needed. */
+        app_settings.show_desktop = FALSE;
     }
 
-    if( files != default_files )
-        g_strfreev( files );
-
-    /* If we need to show desktop icons */
-    if( appSettings.showDesktop )
+    /* handle the parsed result of command line args */
+    if( daemon_mode || app_settings.show_desktop )
     {
-        if( G_LIKELY( ! main_window ) )
-            init_folder();
-        fm_desktop_init();
-    }
+        init_folder();
+        run = TRUE; /* we always need to run the main loop for daemon mode */
 
-    if( G_LIKELY( main_window || appSettings.showDesktop ) )
-    {
-        gtk_main(); /* if folder windows or desktop icons are showed. */
+        /* FIXME: are these necessary?? */
+        signal( SIGPIPE, SIG_IGN );
+        signal( SIGHUP, gtk_main_quit );
+        signal( SIGINT, gtk_main_quit );
+        signal( SIGTERM, gtk_main_quit );
     }
     else
+        run = handle_parsed_commandline_args();
+
+    if( app_settings.show_desktop )
     {
-        vfs_mime_type_clean();
-        vfs_file_monitor_clean();
-        /* if ( appSettings.singleInstance ) */
-        single_instance_finalize();
-        free_settings();
-        return 0;
+        fm_turn_on_desktop_icons();
     }
 
-    /* if ( appSettings.singleInstance ) */
+    if( run )   /* run the main loop */
+        gtk_main();
+
     single_instance_finalize();
 
+    if( app_settings.show_desktop )
+        fm_turn_off_desktop_icons();
+
     if( no_desktop )    /* desktop icons is temporarily supressed */
     {
         if( old_show_desktop )  /* restore original settings */
         {
-            old_show_desktop = appSettings.showDesktop;
-            appSettings.showDesktop = TRUE;
+            old_show_desktop = app_settings.show_desktop;
+            app_settings.show_desktop = TRUE;
         }
     }
-    save_settings();    /* write config file */
+
+    if( run )
+        save_settings();    /* write config file */
+
     free_settings();
 
     vfs_volume_finalize();
     vfs_mime_type_clean();
     vfs_file_monitor_clean();
 
-    if( old_show_desktop )   /* If desktop icons are showed, finalize them. */
-        fm_desktop_cleanup();
-
     return 0;
 }
 
